import { Layer as Layer$1, Stage, Group as Group$1 } from 'react-konva';
import React, { useReducer, forwardRef, useRef, useEffect, useImperativeHandle, createContext, useMemo, useContext, useLayoutEffect, useCallback } from 'react';

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var Z_INDEX = 0;
var PORTAL_LAYER_ID = 'react-konva-portal';
var ERROR_MESSAGE = 'You should wrap your root component with <Stage /> from "react-konva-portal" in order for Portals to work';
function useForceRender() {
  var _useReducer = useReducer(function (i) {
    return i + 1;
  }, 0),
      forceRender = _useReducer[1];

  return forceRender;
}
function zIndexComparator(a, b) {
  var _a$zIndex = a.zIndex,
      zIndexA = _a$zIndex === void 0 ? Z_INDEX : _a$zIndex;
  var _b$zIndex = b.zIndex,
      zIndexB = _b$zIndex === void 0 ? Z_INDEX : _b$zIndex;
  if (zIndexA === zIndexB) return 0;
  return zIndexA > zIndexB ? 1 : -1;
}
function mountCmd(key, zIndex, children) {
  return {
    type: 'mount',
    key: key,
    zIndex: zIndex,
    children: children
  };
}
function updateCmd(key, zIndex, children) {
  return {
    type: 'update',
    key: key,
    zIndex: zIndex,
    children: children
  };
}
function unmountCmd(key) {
  return {
    type: 'unmount',
    key: key
  };
}
function notUnmountByKey(key) {
  return function (item) {
    return item.key === key && item.type !== 'unmount';
  };
}
function warnIfDev() {
  if (process.env.NODE_ENV === 'development') {
    var _console;

    (_console = console).warn.apply(_console, arguments);
  }
}

function ManagerComponent(_, ref) {
  var portalsRef = useRef([]);
  var portalsByIdRef = useRef({});
  var unmountedRef = useRef(false);
  var forceRender = useForceRender();
  useEffect(function () {
    return function () {
      unmountedRef.current = true;
    };
  }, []);
  useImperativeHandle(ref, function () {
    function mount(_ref) {
      var meta = _objectWithoutPropertiesLoose(_ref, ["type"]);

      portalsByIdRef.current[meta.key] = meta;
      portalsRef.current.push(meta);
      portalsRef.current = portalsRef.current.sort(zIndexComparator);
    }

    function update(cmd) {
      var target = portalsByIdRef.current[cmd.key];

      if (target) {
        var resort = target.zIndex !== cmd.zIndex;
        target.zIndex = cmd.zIndex;
        target.children = cmd.children;
        if (resort) portalsRef.current = portalsRef.current.sort(zIndexComparator);
      }
    }

    function unmount(cmd) {
      delete portalsByIdRef.current[cmd.key];
      portalsRef.current = portalsRef.current.filter(function (item) {
        return item.key !== cmd.key;
      });
    }

    var handlers = {
      mount: mount,
      unmount: unmount,
      update: update
    };

    function handleCommand(cmd) {
      handlers[cmd.type].call(null, cmd);
    }

    function commitUpdate() {
      if (!unmountedRef.current) forceRender();
    }

    return {
      handle: function handle(input) {
        if (Array.isArray(input)) input.forEach(handleCommand);else handleCommand(input);
        commitUpdate();
      }
    };
  }, [forceRender]);
  return portalsRef.current.map(function (_ref2) {
    var key = _ref2.key,
        children = _ref2.children;
    return React.createElement(React.Fragment, {
      key: key
    }, children);
  });
}

var PortalManager = forwardRef(ManagerComponent);

var StageContext = createContext(null);

function Provider(_ref) {
  var mount = _ref.mount,
      unmount = _ref.unmount,
      update = _ref.update,
      children = _ref.children,
      addManager = _ref.addManager,
      removeManager = _ref.removeManager;
  var value = useMemo(function () {
    var context = {
      mount: mount,
      unmount: unmount,
      update: update,
      addManager: addManager,
      removeManager: removeManager
    };
    Object.defineProperty(context, '__type', {
      value: TYPE_MARKER,
      enumerable: false,
      writable: false
    });
    return context;
  }, [mount, unmount, update, addManager, removeManager]);
  return React.createElement(StageContext.Provider, {
    value: value
  }, children);
}

var TYPE_MARKER = Symbol();

function isStageContext(val) {
  return val && typeof val === 'object' && val['__type'] === TYPE_MARKER;
}

function useStageContext() {
  var value = useContext(StageContext);
  if (!isStageContext(value)) throw new Error(ERROR_MESSAGE);
  return value;
}

function createPortalContainer(Component, auditName) {
  function PortalContainer(props, ref) {
    var id = props.id,
        children = props.children;
    var konvaRef = useRef(null);
    var managerRef = useRef(null);
    var stage = useStageContext();
    var Konva = Component;
    useImperativeHandle(ref, function () {
      return konvaRef.current;
    }, [konvaRef.current]);
    useLayoutEffect(function () {
      if (id != null && managerRef.current) stage === null || stage === void 0 ? void 0 : stage.addManager(auditName, id, managerRef.current);
      return function () {
        if (id != null) stage === null || stage === void 0 ? void 0 : stage.removeManager(auditName, id);
      };
    }, [id, stage, auditName]);
    return React.createElement(Konva, Object.assign({}, props, {
      id: id,
      ref: konvaRef
    }), children, React.createElement(PortalManager, {
      id: id,
      ref: managerRef,
      auditName: auditName,
      key: id
    }));
  }

  return forwardRef(PortalContainer);
}

var PortalState;

(function (PortalState) {
  PortalState[PortalState["WILL_MOUNT"] = 0] = "WILL_MOUNT";
  PortalState[PortalState["DID_MOUNT"] = 1] = "DID_MOUNT";
  PortalState[PortalState["WILL_UNMOUNT"] = 2] = "WILL_UNMOUNT";
})(PortalState || (PortalState = {}));

function Transporter(_ref) {
  var children = _ref.children,
      _ref$containerId = _ref.containerId,
      containerId = _ref$containerId === void 0 ? PORTAL_LAYER_ID : _ref$containerId,
      _ref$zIndex = _ref.zIndex,
      zIndex = _ref$zIndex === void 0 ? Z_INDEX : _ref$zIndex;
  var stage = useStageContext();
  var keyRef = useRef(-1);
  var phaseRef = useRef(PortalState.WILL_MOUNT);
  useLayoutEffect(function () {
    return function () {
      phaseRef.current = PortalState.WILL_UNMOUNT;
    };
  }, []);
  useLayoutEffect(function () {
    if (phaseRef.current === PortalState.WILL_MOUNT) {
      keyRef.current = stage === null || stage === void 0 ? void 0 : stage.mount(containerId, zIndex, children);
      phaseRef.current = PortalState.DID_MOUNT;
    }

    if (phaseRef.current === PortalState.DID_MOUNT) {
      stage === null || stage === void 0 ? void 0 : stage.update(containerId, keyRef.current, zIndex, children);
    }

    return function () {
      if (phaseRef.current === PortalState.WILL_UNMOUNT) {
        stage === null || stage === void 0 ? void 0 : stage.unmount(containerId, keyRef.current);
      }
    };
  }, [stage, containerId, zIndex, children]);
  return null;
}

function Portal(_ref2) {
  var containerId = _ref2.containerId,
      props = _objectWithoutPropertiesLoose(_ref2, ["containerId"]);

  return React.createElement(Transporter, Object.assign({}, props, {
    key: containerId,
    containerId: containerId
  }));
}

var PortalLayer = createPortalContainer(Layer$1, 'Layer');

function StageComponent(_ref, ref) {
  var children = _ref.children,
      portalLayerProps = _ref.portalLayerProps,
      props = _objectWithoutPropertiesLoose(_ref, ["children", "portalLayerProps"]);

  var seqRef = useRef(0);
  var stageRef = useRef(null);
  var queueRef = useRef({});
  var delayRef = useRef(true);
  var managersRef = useRef({});
  useImperativeHandle(ref, function () {
    return stageRef.current;
  }, [stageRef.current]);
  useLayoutEffect(function () {
    Object.keys(queueRef.current).forEach(function (id) {
      var _managersRef$current$;

      return (_managersRef$current$ = managersRef.current[id]) === null || _managersRef$current$ === void 0 ? void 0 : _managersRef$current$.handle(queueRef.current[id]);
    });
    delayRef.current = false;
    queueRef.current = {};
  }, []);
  var addManager = useCallback(function (auditName, id, manager) {
    if (managersRef.current[id]) {
      warnIfDev("Found " + auditName + " with duplicate id \"" + id + "\".");
    }

    managersRef.current[id] = manager;
  }, []);
  var removeManager = useCallback(function (auditName, id) {
    if (!managersRef.current[id]) {
      warnIfDev("You are trying to remove unknown " + auditName + " instance with id \"" + id + "\".");
    }

    delete managersRef.current[id];
  }, []);
  var mount = useCallback(function (id, zIndex, children) {
    var key = ++seqRef.current;
    var managerReady = managersRef.current[id];
    if (managerReady && !delayRef.current) managerReady.handle(mountCmd(key, zIndex, children));else {
      var queue = queueRef.current[id] || [];
      queue.push(mountCmd(key, zIndex, children));
      queueRef.current[id] = queue;
    }
    return key;
  }, []);
  var update = useCallback(function (id, key, zIndex, children) {
    var managerReady = managersRef.current[id];
    if (managerReady && !delayRef.current) managerReady.handle(updateCmd(key, zIndex, children));else {
      var queue = queueRef.current[id] || [];
      var index = queue.findIndex(notUnmountByKey(key));
      if (index === -1) queue.push(mountCmd(key, zIndex, children));else queue.splice(index, 1, mountCmd(key, zIndex, children));
      queueRef.current[id] = queue;
    }
  }, []);
  var unmount = useCallback(function (id, key) {
    var managerReady = managersRef.current[id];
    if (managerReady && !delayRef.current) managerReady.handle(unmountCmd(key));else {
      var queue = queueRef.current[id] || [];
      queue.push(unmountCmd(key));
      queueRef.current[id] = queue;
    }
  }, []);
  return React.createElement(Stage, Object.assign({
    ref: stageRef
  }, props), React.createElement(Provider, {
    mount: mount,
    update: update,
    unmount: unmount,
    addManager: addManager,
    removeManager: removeManager
  }, children, React.createElement(PortalLayer, Object.assign({}, portalLayerProps, {
    id: PORTAL_LAYER_ID
  }))));
}

var PortalStage = forwardRef(StageComponent);

var Group = createPortalContainer(Group$1, 'Group');
var Layer = createPortalContainer(Layer$1, 'Layer');

export { Group, Layer, PORTAL_LAYER_ID, Portal, PortalStage as Stage, Z_INDEX };
//# sourceMappingURL=index.modern.js.map
